# -*- coding: utf-8 -*-

"""Utilities for interacting with Slack."""

import logging
from functools import lru_cache
from typing import Optional

import pystow
from slack_sdk import WebClient
from slack_sdk.errors import SlackApiError

__all__ = [
    "post",
]

logger = logging.getLogger(__name__)


@lru_cache(maxsize=1)
def _get_client(token: Optional[str] = None) -> Optional[WebClient]:
    token = pystow.get_config("bioversions", "slack_api_token", passthrough=token)
    if token is None:
        return None
    return WebClient(token=token)


def post(text: str, channel: str = "random", token: Optional[str] = None):
    """Post the message to a given Slack channel."""
    client = _get_client(token)
    if client is None:
        return None

    if not channel.startswith("#"):
        channel = f"#{channel}"
    try:
        response = client.chat_postMessage(
            channel=channel,
            text=text,
        )
    except SlackApiError as e:
        # You will get a SlackApiError if "ok" is False
        if not e.response["ok"]:
            raise ValueError('Response is not "ok"') from e
        logger.warning(f"Got an error: {e.response['error']}")
    else:
        return response


if __name__ == "__main__":
    post("Slack test!")



# -*- coding: utf-8 -*-

"""A getter for DrugBank."""

from operator import itemgetter

import requests

from bioversions.utils import Getter, VersionType

__all__ = [
    "DrugBankGetter",
]

URL = "https://go.drugbank.com/releases.json"


class DrugBankGetter(Getter):
    """A getter for DrugBank."""

    bioregistry_id = "drugbank"
    name = "DrugBank"
    homepage_fmt = "https://go.drugbank.com/releases/{version}"
    date_fmt = "%Y-%m-%d"
    version_type = VersionType.semver

    def get(self):
        """Get the latest DrugBank version number."""
        res = requests.get(URL)
        res.raise_for_status()
        latest = max(res.json(), key=itemgetter("released_on"))
        return dict(date=latest["released_on"], version=latest["version"])

    @staticmethod
    def homepage_version_transform(version: str) -> str:
        """Replace dots with dashes for DrugBank homepage format."""
        return version.replace(".", "-")


if __name__ == "__main__":
    DrugBankGetter.print()



# -*- coding: utf-8 -*-

"""A getter for KEGG."""

from typing import Mapping

from bioversions.utils import Getter, VersionType, get_soup

__all__ = [
    "KEGGGetter",
]

URL = "https://www.kegg.jp/kegg/docs/relnote.html"


class KEGGGetter(Getter):
    """A getter for KEGG."""

    bioregistry_id = "kegg.pathway"
    name = "KEGG"
    date_fmt = "%B %d, %Y"
    version_type = VersionType.semver_minor

    def get(self) -> Mapping[str, str]:
        """Get the latest KEGG version number."""
        soup = get_soup(URL)
        header = soup.find("h4")
        sibling = header.next_sibling.strip()
        version, date = [part.strip() for part in sibling.split(",", 1)]
        version = version[len("Release ") :]
        return dict(version=version, date=date)


if __name__ == "__main__":
    KEGGGetter.print()



# -*- coding: utf-8 -*-

"""A getter for FlyBase."""

import ftplib

from bioversions.utils import Getter, VersionType

__all__ = [
    "FlybaseGetter",
]


class FlybaseGetter(Getter):
    """A getter for FlyBase."""

    bioregistry_id = "flybase"
    name = "FlyBase"
    homepage_fmt = "http://ftp.flybase.net/releases/FB{version}/"
    version_type = VersionType.date

    def get(self):
        """Get the latest flybase version number."""
        with ftplib.FTP("ftp.flybase.net") as ftp:
            ftp.login()
            ftp.cwd("releases")
            for name in sorted(ftp.nlst(), reverse=True):
                if name.startswith("FB2"):
                    return name[len("FB") :]
        raise ValueError


if __name__ == "__main__":
    FlybaseGetter.print()
